#include <iostream>
#include <fstream>
#include <algorithm>
#include <array>
#include <vector>
#include <queue>
#include <stack>
#include <ctime>
#include <cstdlib>
typedef unsigned int u32;
typedef unsigned long long u64;
std::size_t myRand(){
    return (std::size_t(rand())<<11) ^ std::size_t(rand());
}
using Table=std::array<std::array<u64, 512>, 300>;
std::size_t n;
u32 graph[300][512];
Table routes;
struct Edge{
    std::size_t a, b;
    u32 len;
};
struct Graph{
    std::array<std::vector<std::size_t>, 300> gr;
    u64 score;
    void add(std::size_t a, std::size_t b){
        gr[a].push_back(b);
        gr[b].push_back(a);
        return;
    }
    void remove(std::size_t a, std::size_t b){
        std::swap(gr[a].back(), *std::find(gr[a].begin(), gr[a].end(), b));
        gr[a].pop_back();
        std::swap(gr[b].back(), *std::find(gr[b].begin(), gr[b].end(), a));
        gr[b].pop_back();
        return;
    }
    std::array<u32, 300> dfs(std::initializer_list<std::size_t> init){
        std::array<u32, 300> path;
        path.fill(-1);
        std::stack<std::size_t> q;
        for(auto i:init){
            path[i]=0;
            q.push(i);
        }
        while(!q.empty()){
            std::size_t i=q.top();
            q.pop();
            for(auto j:gr[i]){
                if(path[j]+1!=0) continue;
                path[j]=path[i]+graph[i][j];
                q.push(j);
            }
        }
        return path;
    }
    u64 scorePartial(std::size_t init){
        auto path=dfs({init});
        u64 res=0;
        for(std::size_t i=0;i<n;i++){
            if(i==init) continue;
            res+=path[i]*routes[init][i];
        }
        return res;
    }
    void rescore(){
        u64 s=0;
        for(std::size_t i=0;i<n;i++){
            s+=scorePartial(i);
        }
        score=s/2;
        return;
    }
    std::vector<std::size_t> path(std::size_t a, std::size_t b){
        std::array<std::size_t, 300> from;
        from.fill(-1);
        std::vector<std::size_t> q;
        q.push_back(b);
        while(from[a]+1==0){
            std::size_t i=q.back();
            q.pop_back();
            for(auto j:gr[i]){
                if(from[j]+1!=0 || j==b) continue;
                from[j]=i;
                q.push_back(j);
            }
        }
        std::vector<std::size_t> res;
        while(a+1!=0){
            res.push_back(a);
            a=from[a];
        }
        return std::move(res);
    }
};
bool operator<(const Graph &a, const Graph &b){
    return a.score<b.score;
}
bool operator==(const Graph &a, const Graph &b){
    return a.score==b.score;
}
struct NodeSet: public Graph{
    Table targetCost, targetSize;
    std::size_t set[300];
    std::vector<std::size_t> in[300];
    void init(){
        for(std::size_t i=0;i<n;i++){
            set[i]=i;
            in[i]={i};
        }
        for(std::size_t i=0;i<n;i++){
            std::fill(targetCost[i].data(), targetCost[i].data()+n, 0);
            for(std::size_t j=0;j<n;j++){
                targetSize[i][j]=routes[i][j];
            }
        }
        return;
    }
    u64 edgeCost(std::size_t a, std::size_t b) const{
        return targetCost[a][set[b]]+targetCost[b][set[a]]+targetSize[set[a]][set[b]]*graph[a][b];
    }
    double edgeRank(std::size_t a, std::size_t b) const{
        return double(targetCost[a][set[b]]+targetCost[b][set[a]])/targetSize[set[a]][set[b]]+graph[a][b];
    }
    void connect(std::size_t a, std::size_t b){
        std::size_t aS=set[a], bS=set[b];
        u32 l=graph[a][b];
        auto path=dfs({a,b});
        auto costA=targetCost[a];
        auto costB=targetCost[b];
        for(std::size_t s=0;s<n;s++){
            if(s==aS || s==bS) continue;
            if(in[s].empty()) continue;
            for(auto i:in[aS]){
                targetCost[i][s]+=(path[i]+l)*targetSize[bS][s]+costB[s];
            }
        }
        for(std::size_t s=0;s<n;s++){
            if(s==aS || s==bS) continue;
            if(in[s].empty()) continue;
            for(auto i:in[bS]){
                targetCost[i][s]+=(path[i]+l)*targetSize[aS][s]+costA[s];
            }
        }
        for(std::size_t i=0;i<n;i++){
            targetCost[i][aS]+=targetCost[i][bS];
            targetCost[i][bS]=0;
        }
        for(std::size_t s=0;s<n;s++){
            if(s==aS || s==bS) continue;
            if(in[s].empty()) continue;
            targetSize[aS][s]+=targetSize[bS][s];
            targetSize[bS][s]=0;
            targetSize[s][aS]+=targetSize[s][bS];
            targetSize[s][bS]=0;
        }
        for(auto i:in[bS]){
            set[i]=aS;
            in[aS].push_back(i);
        }
        in[bS].clear();
        add(a, b);
        return;
    }
};
struct UnionFind{
    std::size_t a[300];
    UnionFind(){
        for(std::size_t i=0;i<n;i++) a[i]=i;
        return;
    }
    std::size_t find(std::size_t i){
        return a[i]==i?i:a[i]=find(a[i]);
    }
    bool union_(std::size_t x, std::size_t y){
        if(find(x)==find(y)) return false;
        a[a[y]]=a[x];
        return true;
    }
};
constexpr std::size_t genSize=12, mutations=6;
std::size_t edgeIx[300][512];
std::vector<std::pair<std::size_t, std::size_t>> edges;
static NodeSet initNodes;
void initial(){
    static bool used[1<<20];
    initNodes.init();
    for(std::size_t i=1;i<n;i++){
        double minR=1.0/0.0;
        std::size_t minE=0;
        for(std::size_t e=0;e<edges.size();e++){
            if(used[e]) continue;
            if(initNodes.set[edges[e].first]==initNodes.set[edges[e].second]){
                used[e]=1;
                continue;
            }
            double r=initNodes.edgeRank(edges[e].first, edges[e].second);
            if(r<minR){
                minR=r;
                minE=e;
            }
        }
        u64 cost=initNodes.edgeCost(edges[minE].first, edges[minE].second);
        initNodes.score+=cost;
        initNodes.connect(edges[minE].first, edges[minE].second);
        used[minE]=1;
    }
    return;
}
int main(){
    std::size_t m;
    std::ifstream fin("graph.in");
    std::ofstream fout("graph.out");
    fin>>n>>m;
    for(std::size_t k=0;k<m;k++){
        std::size_t i, j;
        u32 l;
        fin>>i>>j>>l;
        i--;
        j--;
        graph[i][j]=l;
        graph[j][i]=l;
        edgeIx[i][j]=k;
        edgeIx[j][i]=k;
        edges.emplace_back(i, j);
    }
    for(std::size_t i=0;i<n;i++){
        for(std::size_t j=0;j<n;j++){
            fin>>routes[i][j];
        }
    }
    std::vector<Graph> generation;
    Graph best;
    best.score=-1;
    generation.reserve(genSize);

    initial();
    if(initNodes.score<best.score) best=initNodes;
    generation.emplace_back(std::move(initNodes));
    
    for(std::size_t i=0;i<genSize;i++){
        Graph p;
        UnionFind uf;
        std::random_shuffle(edges.begin(), edges.end());
        for(auto i:edges){
            if(uf.union_(i.first, i.second)){
                p.add(i.first, i.second);
            }
        }
        p.rescore();
        if(p.score<best.score) best=p;
        generation.emplace_back(std::move(p));
    }

    std::sort(generation.begin(), generation.end());
    generation.erase(std::unique(generation.begin(), generation.end()), generation.end());
    if(generation.size()>genSize) generation.resize(genSize);
    
    while(true){
        std::vector<Graph> g2;
        std::size_t k=0;
        for(auto &p:generation){
            if(clock()>15*CLOCKS_PER_SEC/8) break;
            for(std::size_t i=0;i<(genSize-k)*mutations;i++){
                std::size_t e=myRand()%edges.size();
                auto path=p.path(edges[e].first, edges[e].second);
                if(path.size()==2){
                    i--;
                    continue;
                }
                std::size_t rem=myRand()%(path.size()-1);
                Graph p2=p;
                p2.remove(path[rem], path[rem+1]);
                p2.add(edges[e].first, edges[e].second);
                p2.rescore();
                if(p2.score<best.score) best=p2;
                g2.emplace_back(std::move(p2));
            }
            k++;
        }
        if(clock()>15*CLOCKS_PER_SEC/8) break;
        std::sort(g2.begin(), g2.end());
        g2.erase(std::unique(g2.begin(), g2.end()), g2.end());
        if(g2.size()>genSize) g2.resize(genSize);
        generation=std::move(g2);
    }
    for(std::size_t i=0;i<n;i++){
        for(auto j:best.gr[i]){
            if(i<j) fout<<edgeIx[i][j]+1<<'\n';
        }
    }
    return 0;
}
